@@ -56,6 +56,7 @@ static void message_add_recipients_from_string (GMimeMessage *message, char *typ
 static void message_set_subject (GMimeMessage *message, const char *subject);
 
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
+static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_msgid (GMimeStream *stream, const char *name, const char *value);
 
 
@@ -132,6 +133,7 @@ g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 	message->message_id = NULL;
 	message->mime_part = NULL;
 	
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
 }
@@ -511,6 +513,22 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 	return nwritten;
 }
 
+static ssize_t
+write_subject (GMimeStream *stream, const char *name, const char *value)
+{
+	char *unfolded, *folded;
+	ssize_t n;
+	
+	unfolded = g_strdup_printf ("%s: %s\n", name, value);
+	folded = g_mime_utils_unstructured_header_fold (unfolded);
+	g_free (unfolded);
+	
+	n = g_mime_stream_write_string (stream, folded);
+	g_free (folded);
+	
+	return n;
+}
+
 static ssize_t
 write_msgid (GMimeStream *stream, const char *name, const char *value)
 {
@@ -741,23 +759,20 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	ssize_t nwritten, total = 0;
 	
 	/* write the content headers */
-	nwritten = g_mime_header_write_to_stream (object->headers, stream);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
 		return -1;
 	
 	total += nwritten;
 	
 	if (message->mime_part) {
-		nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
-		if (nwritten == -1)
+		if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
 			return -1;
 		
 		total += nwritten;
 		
 		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
 	} else {
-		nwritten = g_mime_stream_write (stream, "\n", 1);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
 			return -1;
 	}
 	
@@ -1277,7 +1292,7 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
  * Write the contents of the MIME Message to @stream.
  *
  * WARNING: This interface is deprecated. Use
- * #g_mime_object_write_to_stream instead.
+ * g_mime_object_write_to_stream() instead.
  *
  * Returns -1 on fail.
  **/
@@ -1298,7 +1313,7 @@ g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
  * Allocates a string buffer containing the mime message @message.
  *
  * WARNING: This interface is deprecated. Use
- * #g_mime_object_to_string instead.
+ * g_mime_object_to_string() instead.
  *
  * Returns an allocated string containing the MIME Message.
  **/
