@@ -119,9 +119,9 @@ struct _GMimeParserPrivate {
 struct _boundary_stack {
 	struct _boundary_stack *parent;
 	unsigned char *boundary;
-	unsigned int boundarylen;
-	unsigned int boundarylenfinal;
-	unsigned int boundarylenmax;
+	size_t boundarylen;
+	size_t boundarylenfinal;
+	size_t boundarylenmax;
 	off_t content_end;
 };
 
@@ -130,7 +130,7 @@ parser_push_boundary (GMimeParser *parser, const char *boundary)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _boundary_stack *s;
-	unsigned int max;
+	size_t max;
 	
 	max = priv->bounds ? priv->bounds->boundarylenmax : 0;
 	
@@ -258,6 +258,7 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
 	parser->priv->persist_stream = TRUE;
+	parser->priv->respect_content_length = FALSE;
 }
 
 static void
@@ -397,11 +398,11 @@ g_mime_parser_new_with_stream (GMimeStream *stream)
  * or resetting of the stream. Anything that will/could change the
  * current stream's offset is PROHIBITED.
  *
- * It is also recommended that you not use #g_mime_stream_tell because
- * it will not necessarily give you the current @parser offset since
- * @parser handles its own internal read-ahead buffer. Instead, it is
- * recommended that you use #g_mime_parser_tell if you have a reason
- * to need the current offset of the @parser.
+ * It is also recommended that you not use g_mime_stream_tell()
+ * because it will not necessarily give you the current @parser offset
+ * since @parser handles its own internal read-ahead buffer. Instead,
+ * it is recommended that you use g_mime_parser_tell() if you have a
+ * reason to need the current offset of the @parser.
  **/
 void
 g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
@@ -611,8 +612,7 @@ parser_fill (GMimeParser *parser)
 	priv->inend = inbuf;
 	inend = priv->realbuf + SCAN_HEAD + SCAN_BUF - 1;
 	
-	nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf);
-	if (nread > 0)
+	if ((nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf)) > 0)
 		priv->inend += nread;
 	
 	priv->offset = g_mime_stream_tell (priv->stream);
@@ -624,6 +624,9 @@ parser_fill (GMimeParser *parser)
 static off_t
 parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
 {
+	if (priv->offset == -1)
+		return -1;
+	
 	if (!inptr)
 		inptr = priv->inptr;
 	
@@ -727,7 +730,7 @@ parser_step_from (GMimeParser *parser)
 
 #define header_backup(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
-		unsigned int hlen, hoff;                                  \
+		size_t hlen, hoff;                                        \
 		                                                          \
 		hlen = hoff = priv->headerptr - priv->headerbuf;          \
 		hlen = hlen ? hlen : 1;                                   \
@@ -748,7 +751,7 @@ parser_step_from (GMimeParser *parser)
 #define header_parse(parser, priv, hend) G_STMT_START {                   \
 	register unsigned char *colon;                                    \
 	struct _header_raw *header;                                       \
-	unsigned int hlen;                                                \
+	size_t hlen;                                                      \
 	                                                                  \
 	header = g_new (struct _header_raw, 1);                           \
 	header->next = NULL;                                              \
@@ -878,8 +881,7 @@ parser_content_type (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	const char *content_type;
 	
-	content_type = header_raw_find (priv->headers, "Content-Type", NULL);
-	if (content_type)
+	if ((content_type = header_raw_find (priv->headers, "Content-Type", NULL)))
 		return g_mime_content_type_new_from_string (content_type);
 	
 	return NULL;
@@ -966,7 +968,7 @@ enum {
 			  (len >= 2 && (start[0] == '-' && start[1] == '-')))
 
 static int
-check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, int len)
+check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, size_t len)
 {
 	off_t offset = parser_offset (priv, (unsigned char *) start);
 	
@@ -1177,14 +1179,14 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _header_raw *header;
 	GMimeObject *object;
+	const char *ctype;
 	
 	/* get the headers */
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
 	if (!content_type) {
-		content_type = parser_content_type (parser);
-		if (!content_type)
+		if (!(content_type = parser_content_type (parser)))
 			content_type = g_mime_content_type_new ("text", "plain");
 	}
 	
@@ -1197,7 +1199,8 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	g_mime_object_set_content_type (object, content_type);
+	g_mime_content_type_destroy (object->content_type);
+	object->content_type = content_type;
 	
 	/* skip empty line after headers */
 	parser_skip_line (parser);
@@ -1303,7 +1306,8 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	g_mime_object_set_content_type (object, content_type);
+	g_mime_content_type_destroy (object->content_type);
+	object->content_type = content_type;
 	
 	multipart = (GMimeMultipart *) object;
 	
@@ -1455,7 +1459,7 @@ g_mime_parser_construct_message (GMimeParser *parser)
  * @parser: MIME parser object
  *
  * Gets the mbox-style From-line of the most recently parsed message
- * (gotten from #g_mime_parser_construct_message).
+ * (gotten from g_mime_parser_construct_message()).
  *
  * Returns the mbox-style From-line of the most recently parsed
  * message or %NULL on error.
@@ -1483,7 +1487,7 @@ g_mime_parser_get_from (GMimeParser *parser)
  * @parser: MIME parser object
  *
  * Gets the offset of the most recently parsed mbox-style From-line
- * (gotten from #g_mime_parser_construct_message).
+ * (gotten from g_mime_parser_construct_message()).
  *
  * Returns the offset of the most recently parsed mbox-style From-line
  * or -1 on error.
